;+
;
; NAME:
;	READ_HXRS_FITS
;
; PURPOSE:
;       Reads hxrs fits files into idl, then
;       computes counts/sec and live times for shielded and non-shielded
;       detectors and effective total livetime for sum of both s. and ns.
;
; CATEGORY:
;       HXRS
;
; CALLING SEQUENCE:
;
; CALLS:
;       CHECKVAR, EDGE_HXRS, F_DIV, ATIME, UTPLOT
;
; INPUTS:
;	Through keywords so you don't have to remember the input order.
;
; OUTPUTS:
;	Through keywords so you don't have to remember the output order.
;
; KEYWORDS:
;        file:	hxrs fits file name to read (input)
;        ut:	time in seconds since 1-jan-1979, middle of datapoint,
;		dblarr(2,n) (start and end of each interval) (output)
;        syarray, nyarray, totyarray: shielded, nonsh and total raw counts/interval
;        esyarray, enyarray,etotyarray: uncertainties, i.e. sqrt(yarray(10,N)
;        deltat:	accumulation time interval, scalar, in seconds (output)
;        slivetime, nlivetime, totlivetime: livetimes per interval for detectors and their total
;        edges:	output energy loss bands in keV  fltarr(2,10) (output)
;        h0:	primary header (string array) (output)
;        h1:	header from first (time) extension (string array) (output)
;        error:	if =1, then error reading file.  Otherwise 0. (output)
;
; PROCEDURE:
;	User may explicitly pass the name of a HXRS FITS file;
; 	Sample call:
;	read_hxrs_fits, file='hxrs_fits:hxrs_23.fits', ut=ut, syarray=syarray
;
; MODIFICATION HISTORY:
;        Adapted from the get_hxrbs_fits routine
;	Written by Kim Tolbert (NASA/GSFC), 9-sep-1994.
;        Adapted for HXRS by Alan Kiplinger May 2000.
;


PRO READ_HXRS_FITS, file=file, ut=ut, syarray=syarray, nyarray=nyarray, $
         totyarray=totyarray, esyarray=esyarray, enyarray=enyarry,      $
         etotyarray=etotyarray, deltat=deltat, slivetime=slivetime,     $
         nlivetime=nlivetime, totlivetime=totlivetime, edges=edges,     $
         h0=h0, h1=h1, error=error

error = 0


if (size(file))(1) eq 0 then begin
   file=' '
   read,' Enter input filename: ',file
endif

; read fits file, primary and first (and only) extension


  datain=mrdfits(file,0,h0)       ; read first header
  datain=mrdfits(file,1,h1)       ; read data structure and
                                  ; header from 1st extension

 ; establish data array and create ut time array for SPEX

  date_obs = STRTRIM(sxpar(h1, 'DATE_OBS'), 2)   ; read startime of 1st interval
  t0 = anytim(date_obs)           ; get time of first interval since Jan. 1, 1979
  syarray=datain.counts_s + 32768 ; get shielded data (raw counts/interval)
  nyarray=datain.counts_n + 32768 ; get non-shielded data (raw counts/interval)
  totyarray=float(syarray+nyarray) ; create array for total of both instruments
  esyarray=sqrt(syarray)           ; compute error arrays
  enyarray=sqrt(nyarray)
  etotyarray=sqrt(totyarray)        ; compute errors for summed array
  times=datain.time               ; get UNIX interval start times (since 1/1/70)
  times=times-times(0)            ; relative seconds since first interval
  nbin = long(n_elements(times))
  utstarts=dblarr(nbin)           ; create time array for interval starting times
  utstarts(0)=times +datain.finetime/1000. + t0 ; add fine time +t0 corrections to UT times

  blocktype=datain.blocktype      ; blocktypes 0 (1s) & 8 (0.2s) are normal and flare modes
  flaremode=where(blocktype eq 8,count8)      ; identify intervals in 0.2s flare mode
  normalmode=where(blocktype eq 0,count0)     ; identify normal 1 s mode data
  nbinflare=long(n_elements(flaremode))

  slivetime=fltarr(nbin)          ; create livetime array for shielded data
  nlivetime=slivetime             ; create livetime array for nonshielded data
  totlivetime=slivetime +1.          ; create effective livetime array for sum of s. + ns.


 for j=long(0),nbin-1 do begin            ; compute livetimes for all data intervals
                                  ; first 8 channels have 4 microsec. per pulse deadtime

   if blocktype(j) eq 8 then slivetime(j)=.2-(total(syarray(0:7,j))*4e-6 +  syarray(8,j)*4e-5) $
      else slivetime(j)=1.-(total(syarray(0:7,j))*4e-6 +  syarray(8,j)*4e-5)

   if blocktype(j) eq 8 then nlivetime(j)=.2-(total(nyarray(0:7,j))*4e-6 +  nyarray(8,j)*4e-5) $
      else nlivetime(j)=1.-(total(nyarray(0:7,j))*4e-6 +  nyarray(8,j)*4e-5)

                  ; now compute effectie livetime for sum of sh and non-sh
    if slivetime(j) gt 0. and nlivetime(j) gt 0. and totyarray(j) ne 0 then begin
      totlivetime(j) = totyarray(j) / ((syarray(j)/slivetime(j)) + (nyarray(j)/nlivetime(j)))
    endif


  endfor


  deltat=dblarr(nbin) +1.         ; create integration time array for each intv. (def=1.0s)

  for i=long(0),nbinflare-1 do begin
     deltat(flaremode(i))=.2      ; reset flaremode intervals to 0.2s
  endfor

  utends=utstarts + deltat        ; create end time array by augmenting start times by deltat
  ut=dblarr(2,nbin)
  ut(0,*)=utstarts                ; create ut array for SPEX
  ut(1,*)=utends


 width = indgen(9)
 nchan = 9

; compute energy channel widths
  edges_hxrs, edges, width=ww

; hxrs central detector area
area = 4.909;cm^2



goto,getout

error_exit:
error=1

getout:
return
end


